"""Module with session for authenticated requests to server."""
import os
import posixpath
from http import HTTPStatus

import requests
from ML_management.mlmanagement.base_exceptions import ClientException
from ML_management.mlmanagement.singleton_pattern import Singleton
from ML_management.mlmanagement.variables import _get_server_ml_api, _get_server_url
from requests import Response
from sgqlc.endpoint.http import HTTPEndpoint
from sgqlc.operation import Operation


class InvalidCredentials(ClientException):
    """Exception for invalid login-password pair."""

    pass


class MissedEnvVar(ClientException):
    """Exception for the case of the absence of the necessary environment variable."""

    pass


class InvalidEnvVar(ClientException):
    """Exception for the case of the invalid value of environment variable."""

    pass


class AuthSession(metaclass=Singleton):
    """Extend the standard session functionality with authentication functionality."""

    def __init__(self) -> None:
        server_url = _get_server_url()
        if server_url is None:
            raise MissedEnvVar("You must specify environment variable server_url.")
        if not server_url.startswith("http"):
            raise InvalidEnvVar("$server_url must starts with http or https.")

        self.cookies = {}
        # check mlflow endpoint is available without authentication
        if requests.get(_get_server_ml_api()).status_code == HTTPStatus.METHOD_NOT_ALLOWED:
            return
        if self._try_set_cookies():
            return
        login, password = os.getenv("login"), os.getenv("password")
        if not self._try_authenticate_by_credentials(login, password):
            raise InvalidCredentials(f"User with login {login} and password {password} does not exist.")

    def get(self, url: str, **kwargs) -> Response:
        """Proxy requests.get."""
        response = requests.get(url, cookies=self.cookies, **kwargs)
        # if token was updated, update our cookie
        self._update_cookies(response, ["kc-access"])
        return response

    def post(self, url: str, **kwargs) -> Response:
        """Proxy requests.post."""
        response = requests.post(url, cookies=self.cookies, **kwargs)
        # if token was updated, update our cookie
        self._update_cookies(response, ["kc-access"])
        return response

    # For sdk auth purposes
    def sgqlc_request(self, operation: Operation) -> dict:
        """Make request to /graphql for operation."""
        cookie_header = "; ".join(f"{cookie_name}={cookie_value}" for cookie_name, cookie_value in self.cookies.items())
        return HTTPEndpoint(posixpath.join(_get_server_url(), "graphql"), base_headers={"Cookie": cookie_header})(operation)

    def _update_cookies(self, response: Response, cookie_names: list) -> None:
        """Update cookies from cookie_names list."""
        for cookie_name in cookie_names:
            if cookie := response.cookies.get(cookie_name):
                self.cookies[cookie_name] = cookie

    def _try_set_cookies(self) -> bool:
        kc_access, kc_state = os.getenv("kc_access"), os.getenv("kc_state")
        if kc_access is not None and kc_state is not None:
            for name, value in zip(["kc-access", "kc-state"], [kc_access, kc_state]):
                self.cookies[name] = value
            return True
        return False

    def _try_authenticate_by_credentials(self, login, password) -> bool:
        if login is None or password is None:
            return False
        response = requests.post(posixpath.join(_get_server_url(), "oauth", "login"), data={"username": login, "password": password})
        if response.status_code == HTTPStatus.UNAUTHORIZED:
            return False
        if response.status_code != HTTPStatus.OK:
            response.raise_for_status()
        self.cookies = {
            "kc-state": response.cookies["kc-state"],
            "kc-access": response.cookies["kc-access"],
        }
        return True



"""Define Model Registry Manager."""
from abc import abstractmethod

from mlflow.entities.model_registry.model_version import ModelVersion
from mlflow.store.entities import PagedList


class AbstractRegistryManager:
    """Abstract Registry Manager to choose necessary version of the model."""

    @abstractmethod
    def get_latest_model_version(
        self,
        *args,
        **kwargs,
    ) -> ModelVersion:
        """Get the latest version of a model "name"."""
        raise NotImplementedError

    @abstractmethod
    def get_latest_dataset_loader_version(
        self,
        *args,
        **kwargs,
    ) -> ModelVersion:
        """Get the latest version of a dataset loader "name"."""
        raise NotImplementedError

    @abstractmethod
    def get_latest_executor_version(
        self,
        *args,
        **kwargs,
    ) -> ModelVersion:
        """Get the latest version of an executor "name"."""
        raise NotImplementedError

    @abstractmethod
    def get_best_model_version(self, *args, **kwargs) -> ModelVersion:
        """Get best version of model "name" according to a metric."""
        raise NotImplementedError

    @abstractmethod
    def get_initial_model_version(self, *args, **kwargs) -> ModelVersion:
        """Get initial version of model "name"."""
        raise NotImplementedError

    @abstractmethod
    def get_initial_dataset_loader_version(self, *args, **kwargs) -> ModelVersion:
        """Get initial version of dataset loader "name"."""
        raise NotImplementedError

    @abstractmethod
    def get_initial_executor_version(self, *args, **kwargs) -> ModelVersion:
        """Get initial version of executor "name"."""
        raise NotImplementedError

    @abstractmethod
    def get_all_model_versions(self, *args, **kwargs) -> PagedList[ModelVersion]:
        """
        Get all versions of a given model and return PagedList of ModelVersion objects.

        https://mlflow.org/docs/latest/python_api/mlflow.entities.html#mlflow.entities.model_registry.ModelVersion

        Parameters:
            name (str): Model name used for model registration.

        Returns:
            PagedList[ModelVersion]: Available model versions
        """
        raise NotImplementedError

    @abstractmethod
    def get_all_dataset_loader_versions(self, *args, **kwargs) -> PagedList[ModelVersion]:
        """
        Get all versions of a given dataset loader and return PagedList of ModelVersion objects.

        https://mlflow.org/docs/latest/python_api/mlflow.entities.html#mlflow.entities.model_registry.ModelVersion

        Parameters:
            name (str): Dataset loader name used for model registration.

        Returns:
            PagedList[ModelVersion]: Available dataset loader versions
        """
        raise NotImplementedError

    @abstractmethod
    def get_all_executor_versions(self, *args, **kwargs) -> PagedList[ModelVersion]:
        """
        Get all versions of a given executor and return PagedList of ModelVersion objects.

        https://mlflow.org/docs/latest/python_api/mlflow.entities.html#mlflow.entities.model_registry.ModelVersion

        Parameters:
            name (str): Executor name used for model registration.

        Returns:
            PagedList[ModelVersion]: Available executor versions
        """
        raise NotImplementedError



import sgqlc.types

schema = sgqlc.types.Schema()


__docformat__ = "markdown"


########################################################################
# Scalars and Enumerations
########################################################################
Boolean = sgqlc.types.Boolean


class ExecutorMethodName(sgqlc.types.Enum):
    """Enumeration Choices:

    * `execute`None
    """

    __schema__ = schema
    __choices__ = ("execute",)


Float = sgqlc.types.Float

ID = sgqlc.types.ID

Int = sgqlc.types.Int


class JSON(sgqlc.types.Scalar):
    """The `JSON` scalar type represents JSON values as specified by
    [ECMA-404](http://www.ecma-
    international.org/publications/files/ECMA-ST/ECMA-404.pdf).
    """

    __schema__ = schema


class JobStatus(sgqlc.types.Enum):
    """Enumeration Choices:

    * `BUILDING`None
    * `EXECUTING`None
    * `FAILED`None
    * `PLANNED`None
    * `SUCCESSFUL`None
    """

    __schema__ = schema
    __choices__ = ("BUILDING", "EXECUTING", "FAILED", "PLANNED", "SUCCESSFUL")


class LONG(sgqlc.types.Scalar):
    """The `LONG` scalar type represents long int type."""

    __schema__ = schema


class ModelMethodName(sgqlc.types.Enum):
    """Enumeration Choices:

    * `evaluate_function`None
    * `finetune_function`None
    * `get_losses`None
    * `get_nn_module`None
    * `get_target_layer`None
    * `predict_function`None
    * `train_function`None
    """

    __schema__ = schema
    __choices__ = (
        "evaluate_function",
        "finetune_function",
        "get_losses",
        "get_nn_module",
        "get_target_layer",
        "predict_function",
        "train_function",
    )


class PeriodicType(sgqlc.types.Enum):
    """Enumeration Choices:

    * `ONCE`None
    * `PERIODIC`None
    """

    __schema__ = schema
    __choices__ = ("ONCE", "PERIODIC")


String = sgqlc.types.String


class UploadModelMode(sgqlc.types.Enum):
    """Enumeration Choices:

    * `new_model`None
    * `new_version`None
    * `none`None
    """

    __schema__ = schema
    __choices__ = ("new_model", "new_version", "none")


class UploadModelType(sgqlc.types.Enum):
    """Enumeration Choices:

    * `new_model`None
    * `new_version`None
    * `root`None
    """

    __schema__ = schema
    __choices__ = ("new_model", "new_version", "root")


########################################################################
# Input Objects
########################################################################
class DataParamsInput(sgqlc.types.Input):
    __schema__ = schema
    __field_names__ = ("dataset_loader_method_params", "collector_method_params")
    dataset_loader_method_params = sgqlc.types.Field(sgqlc.types.non_null("MethodParamsInput"), graphql_name="datasetLoaderMethodParams")

    collector_method_params = sgqlc.types.Field(sgqlc.types.non_null("MethodParamsInput"), graphql_name="collectorMethodParams")


class ExecutorModelParamsInput(sgqlc.types.Input):
    __schema__ = schema
    __field_names__ = ("executor_method_params", "model_methods_params")
    executor_method_params = sgqlc.types.Field(sgqlc.types.non_null("MethodParamsInput"), graphql_name="executorMethodParams")

    model_methods_params = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null("MethodParamsInput"))), graphql_name="modelMethodsParams"
    )


class MethodParamsInput(sgqlc.types.Input):
    __schema__ = schema
    __field_names__ = ("method_name", "method_params")
    method_name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="methodName")

    method_params = sgqlc.types.Field(sgqlc.types.non_null(JSON), graphql_name="methodParams")


class ObjectVersionInput(sgqlc.types.Input):
    __schema__ = schema
    __field_names__ = ("name", "version")
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="name")

    version = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name="version")


class TimestampInterval(sgqlc.types.Input):
    __schema__ = schema
    __field_names__ = ("start", "end")
    start = sgqlc.types.Field(LONG, graphql_name="start")

    end = sgqlc.types.Field(LONG, graphql_name="end")


########################################################################
# Output Objects and Interfaces
########################################################################
class DataParams(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ("dataset_loader_method_params", "collector_method_params")
    dataset_loader_method_params = sgqlc.types.Field(sgqlc.types.non_null("MethodParams"), graphql_name="datasetLoaderMethodParams")

    collector_method_params = sgqlc.types.Field(sgqlc.types.non_null("MethodParams"), graphql_name="collectorMethodParams")


class DataSchema(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ("dataset_loader_method_schema", "collector_method_schema")
    dataset_loader_method_schema = sgqlc.types.Field(sgqlc.types.non_null("MethodSchema"), graphql_name="datasetLoaderMethodSchema")

    collector_method_schema = sgqlc.types.Field(sgqlc.types.non_null("MethodSchema"), graphql_name="collectorMethodSchema")


class DatasetLoader(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = (
        "name",
        "description",
        "owner_id",
        "tags",
        "creation_timestamp",
        "last_updated_timestamp",
        "owner",
        "latest_dataset_loader_version",
        "init_dataset_loader_version",
        "list_dataset_loader_version",
        "pagination_dataset_loader_version",
    )
    name = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name="name")

    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="description")

    owner_id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name="ownerId")

    tags = sgqlc.types.Field(sgqlc.types.non_null(JSON), graphql_name="tags")

    creation_timestamp = sgqlc.types.Field(sgqlc.types.non_null(LONG), graphql_name="creationTimestamp")

    last_updated_timestamp = sgqlc.types.Field(LONG, graphql_name="lastUpdatedTimestamp")

    owner = sgqlc.types.Field(sgqlc.types.non_null("User"), graphql_name="owner")

    latest_dataset_loader_version = sgqlc.types.Field(
        sgqlc.types.non_null("DatasetLoaderVersion"), graphql_name="latestDatasetLoaderVersion"
    )

    init_dataset_loader_version = sgqlc.types.Field(sgqlc.types.non_null("DatasetLoaderVersion"), graphql_name="initDatasetLoaderVersion")

    list_dataset_loader_version = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null("DatasetLoaderVersion"))), graphql_name="listDatasetLoaderVersion"
    )

    pagination_dataset_loader_version = sgqlc.types.Field(
        sgqlc.types.non_null("DatasetLoaderVersionPagination"),
        graphql_name="paginationDatasetLoaderVersion",
        args=sgqlc.types.ArgDict(
            (
                ("limit", sgqlc.types.Arg(Int, graphql_name="limit", default=None)),
                ("offset", sgqlc.types.Arg(Int, graphql_name="offset", default=None)),
            )
        ),
    )
    """Arguments:

    * `limit` (`Int`)None (default: `null`)
    * `offset` (`Int`)None (default: `null`)
    """


class DatasetLoaderPagination(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ("list_dataset_loader", "total")
    list_dataset_loader = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(DatasetLoader))), graphql_name="listDatasetLoader"
    )

    total = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name="total")


class DatasetLoaderVersion(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = (
        "name",
        "version",
        "run_id",
        "source_path",
        "tags",
        "description",
        "status",
        "status_message",
        "creation_timestamp",
        "last_updated_timestamp",
        "dataset_loader_method_schema",
        "dataset_loader_method_schema_name",
        "run",
        "dataset_loader",
        "data_json_schema",
        "list_deployed_jobs",
        "pagination_deployed_jobs",
    )
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="name")

    version = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name="version")

    run_id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name="runId")

    source_path = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="sourcePath")

    tags = sgqlc.types.Field(sgqlc.types.non_null(JSON), graphql_name="tags")

    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="description")

    status = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="status")

    status_message = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="statusMessage")

    creation_timestamp = sgqlc.types.Field(sgqlc.types.non_null(LONG), graphql_name="creationTimestamp")

    last_updated_timestamp = sgqlc.types.Field(LONG, graphql_name="lastUpdatedTimestamp")

    dataset_loader_method_schema = sgqlc.types.Field(sgqlc.types.non_null(JSON), graphql_name="datasetLoaderMethodSchema")

    dataset_loader_method_schema_name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="datasetLoaderMethodSchemaName")

    run = sgqlc.types.Field(sgqlc.types.non_null("Run"), graphql_name="run")

    dataset_loader = sgqlc.types.Field(sgqlc.types.non_null(DatasetLoader), graphql_name="datasetLoader")

    data_json_schema = sgqlc.types.Field(
        sgqlc.types.non_null(DataSchema),
        graphql_name="dataJsonSchema",
        args=sgqlc.types.ArgDict(
            (("collector_name", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="collectorName", default=None)),)
        ),
    )
    """Arguments:

    * `collector_name` (`String!`)None
    """

    list_deployed_jobs = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null("Job"))), graphql_name="listDeployedJobs"
    )

    pagination_deployed_jobs = sgqlc.types.Field(
        sgqlc.types.non_null("JobPagination"),
        graphql_name="paginationDeployedJobs",
        args=sgqlc.types.ArgDict(
            (
                ("limit", sgqlc.types.Arg(Int, graphql_name="limit", default=None)),
                ("offset", sgqlc.types.Arg(Int, graphql_name="offset", default=None)),
            )
        ),
    )
    """Arguments:

    * `limit` (`Int`)None (default: `null`)
    * `offset` (`Int`)None (default: `null`)
    """


class DatasetLoaderVersionPagination(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ("list_dataset_loader_version", "total")
    list_dataset_loader_version = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(DatasetLoaderVersion))), graphql_name="listDatasetLoaderVersion"
    )

    total = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name="total")


class Executor(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = (
        "name",
        "description",
        "owner_id",
        "tags",
        "creation_timestamp",
        "last_updated_timestamp",
        "owner",
        "latest_executor_version",
        "init_executor_version",
        "list_executor_version",
        "pagination_executor_version",
    )
    name = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name="name")

    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="description")

    owner_id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name="ownerId")

    tags = sgqlc.types.Field(sgqlc.types.non_null(JSON), graphql_name="tags")

    creation_timestamp = sgqlc.types.Field(sgqlc.types.non_null(LONG), graphql_name="creationTimestamp")

    last_updated_timestamp = sgqlc.types.Field(LONG, graphql_name="lastUpdatedTimestamp")

    owner = sgqlc.types.Field(sgqlc.types.non_null("User"), graphql_name="owner")

    latest_executor_version = sgqlc.types.Field(sgqlc.types.non_null("ExecutorVersion"), graphql_name="latestExecutorVersion")

    init_executor_version = sgqlc.types.Field(sgqlc.types.non_null("ExecutorVersion"), graphql_name="initExecutorVersion")

    list_executor_version = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null("ExecutorVersion"))), graphql_name="listExecutorVersion"
    )

    pagination_executor_version = sgqlc.types.Field(
        sgqlc.types.non_null("ExecutorVersionPagination"),
        graphql_name="paginationExecutorVersion",
        args=sgqlc.types.ArgDict(
            (
                ("limit", sgqlc.types.Arg(Int, graphql_name="limit", default=None)),
                ("offset", sgqlc.types.Arg(Int, graphql_name="offset", default=None)),
            )
        ),
    )
    """Arguments:

    * `limit` (`Int`)None (default: `null`)
    * `offset` (`Int`)None (default: `null`)
    """


class ExecutorModelParams(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ("executor_method_params", "model_methods_params")
    executor_method_params = sgqlc.types.Field(sgqlc.types.non_null("MethodParams"), graphql_name="executorMethodParams")

    model_methods_params = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null("MethodParams"))), graphql_name="modelMethodsParams"
    )


class ExecutorPagination(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ("list_executor", "total")
    list_executor = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(Executor))), graphql_name="listExecutor"
    )

    total = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name="total")


class ExecutorVersion(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = (
        "name",
        "version",
        "run_id",
        "source_path",
        "tags",
        "description",
        "status",
        "status_message",
        "creation_timestamp",
        "last_updated_timestamp",
        "executor_method_schema",
        "executor_method_schema_name",
        "desired_model_methods",
        "desired_model_patterns",
        "upload_model_mode",
        "run",
        "executor",
        "list_deployed_jobs",
        "pagination_deployed_jobs",
    )
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="name")

    version = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name="version")

    run_id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name="runId")

    source_path = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="sourcePath")

    tags = sgqlc.types.Field(sgqlc.types.non_null(JSON), graphql_name="tags")

    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="description")

    status = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="status")

    status_message = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="statusMessage")

    creation_timestamp = sgqlc.types.Field(sgqlc.types.non_null(LONG), graphql_name="creationTimestamp")

    last_updated_timestamp = sgqlc.types.Field(LONG, graphql_name="lastUpdatedTimestamp")

    executor_method_schema = sgqlc.types.Field(sgqlc.types.non_null(JSON), graphql_name="executorMethodSchema")

    executor_method_schema_name = sgqlc.types.Field(sgqlc.types.non_null(ExecutorMethodName), graphql_name="executorMethodSchemaName")

    desired_model_methods = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(ModelMethodName))), graphql_name="desiredModelMethods"
    )

    desired_model_patterns = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(String))), graphql_name="desiredModelPatterns"
    )

    upload_model_mode = sgqlc.types.Field(sgqlc.types.non_null(UploadModelMode), graphql_name="uploadModelMode")

    run = sgqlc.types.Field(sgqlc.types.non_null("Run"), graphql_name="run")

    executor = sgqlc.types.Field(sgqlc.types.non_null(Executor), graphql_name="executor")

    list_deployed_jobs = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null("Job"))), graphql_name="listDeployedJobs"
    )

    pagination_deployed_jobs = sgqlc.types.Field(
        sgqlc.types.non_null("JobPagination"),
        graphql_name="paginationDeployedJobs",
        args=sgqlc.types.ArgDict(
            (
                ("limit", sgqlc.types.Arg(Int, graphql_name="limit", default=None)),
                ("offset", sgqlc.types.Arg(Int, graphql_name="offset", default=None)),
            )
        ),
    )
    """Arguments:

    * `limit` (`Int`)None (default: `null`)
    * `offset` (`Int`)None (default: `null`)
    """


class ExecutorVersionPagination(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ("list_executor_version", "total")
    list_executor_version = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(ExecutorVersion))), graphql_name="listExecutorVersion"
    )

    total = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name="total")


class Experiment(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ("name", "experiment_id", "artifact_location", "lifecycle_stage", "list_run", "pagination_run", "tags")
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="name")

    experiment_id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name="experimentId")

    artifact_location = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="artifactLocation")

    lifecycle_stage = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="lifecycleStage")

    list_run = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null("Run"))), graphql_name="listRun")

    pagination_run = sgqlc.types.Field(
        sgqlc.types.non_null("RunPagination"),
        graphql_name="paginationRun",
        args=sgqlc.types.ArgDict(
            (
                ("limit", sgqlc.types.Arg(Int, graphql_name="limit", default=None)),
                ("offset", sgqlc.types.Arg(Int, graphql_name="offset", default=None)),
            )
        ),
    )
    """Arguments:

    * `limit` (`Int`)None (default: `null`)
    * `offset` (`Int`)None (default: `null`)
    """

    tags = sgqlc.types.Field(sgqlc.types.non_null(JSON), graphql_name="tags")


class ExperimentPagination(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ("list_experiment", "total")
    list_experiment = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(Experiment))), graphql_name="listExperiment"
    )

    total = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name="total")


class GraphNode(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = (
        "run_id",
        "name",
        "version",
        "source_run_id",
        "source_name",
        "source_version",
        "root_run_id",
        "upload_model_type",
        "list_next_node",
    )
    run_id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name="runId")

    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="name")

    version = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name="version")

    source_run_id = sgqlc.types.Field(ID, graphql_name="sourceRunId")

    source_name = sgqlc.types.Field(String, graphql_name="sourceName")

    source_version = sgqlc.types.Field(String, graphql_name="sourceVersion")

    root_run_id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name="rootRunId")

    upload_model_type = sgqlc.types.Field(sgqlc.types.non_null(UploadModelType), graphql_name="uploadModelType")

    list_next_node = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null("GraphNode")), graphql_name="listNextNode")


class Job(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = (
        "name",
        "owner_id",
        "periodic_type",
        "status",
        "experiment_name",
        "registration_timestamp",
        "start_timestamp",
        "end_timestamp",
        "start_build_timestamp",
        "end_build_timestamp",
        "exception",
        "exception_traceback",
        "job_run_id",
        "init_model_version_run_id",
        "dataset_loader_version_run_id",
        "executor_version_run_id",
        "params",
        "run",
        "experiment",
        "init_model_version",
        "dataset_loader_version",
        "executor_version",
        "result_model_version",
    )
    name = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name="name")

    owner_id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name="ownerId")

    periodic_type = sgqlc.types.Field(sgqlc.types.non_null(PeriodicType), graphql_name="periodicType")

    status = sgqlc.types.Field(sgqlc.types.non_null(JobStatus), graphql_name="status")

    experiment_name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="experimentName")

    registration_timestamp = sgqlc.types.Field(sgqlc.types.non_null(LONG), graphql_name="registrationTimestamp")

    start_timestamp = sgqlc.types.Field(LONG, graphql_name="startTimestamp")

    end_timestamp = sgqlc.types.Field(LONG, graphql_name="endTimestamp")

    start_build_timestamp = sgqlc.types.Field(LONG, graphql_name="startBuildTimestamp")

    end_build_timestamp = sgqlc.types.Field(LONG, graphql_name="endBuildTimestamp")

    exception = sgqlc.types.Field(String, graphql_name="exception")

    exception_traceback = sgqlc.types.Field(String, graphql_name="exceptionTraceback")

    job_run_id = sgqlc.types.Field(String, graphql_name="jobRunId")

    init_model_version_run_id = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="initModelVersionRunId")

    dataset_loader_version_run_id = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="datasetLoaderVersionRunId")

    executor_version_run_id = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="executorVersionRunId")

    params = sgqlc.types.Field(sgqlc.types.non_null("JobParams"), graphql_name="params")

    run = sgqlc.types.Field("Run", graphql_name="run")

    experiment = sgqlc.types.Field(Experiment, graphql_name="experiment")

    init_model_version = sgqlc.types.Field(sgqlc.types.non_null("ModelVersion"), graphql_name="initModelVersion")

    dataset_loader_version = sgqlc.types.Field(sgqlc.types.non_null(DatasetLoaderVersion), graphql_name="datasetLoaderVersion")

    executor_version = sgqlc.types.Field(sgqlc.types.non_null(ExecutorVersion), graphql_name="executorVersion")

    result_model_version = sgqlc.types.Field("ModelVersion", graphql_name="resultModelVersion")


class JobPagination(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ("list_job", "total")
    list_job = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(Job))), graphql_name="listJob")

    total = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name="total")


class JobParams(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = (
        "name",
        "job_executor_name",
        "job_executor_version",
        "dataset_loader_name",
        "dataset_loader_version",
        "model_name",
        "model_version",
        "new_model_name",
        "experiment_name",
        "cron_expression",
        "periodic_type",
        "metric_name",
        "optimal_min",
        "choice_criteria",
        "executor_model_params",
        "data_params",
        "collector_name",
    )
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="name")

    job_executor_name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="jobExecutorName")

    job_executor_version = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name="jobExecutorVersion")

    dataset_loader_name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="datasetLoaderName")

    dataset_loader_version = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name="datasetLoaderVersion")

    model_name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="modelName")

    model_version = sgqlc.types.Field(Int, graphql_name="modelVersion")

    new_model_name = sgqlc.types.Field(String, graphql_name="newModelName")

    experiment_name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="experimentName")

    cron_expression = sgqlc.types.Field(String, graphql_name="cronExpression")

    periodic_type = sgqlc.types.Field(sgqlc.types.non_null(PeriodicType), graphql_name="periodicType")

    metric_name = sgqlc.types.Field(String, graphql_name="metricName")

    optimal_min = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name="optimalMin")

    choice_criteria = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="choiceCriteria")

    executor_model_params = sgqlc.types.Field(sgqlc.types.non_null(ExecutorModelParams), graphql_name="executorModelParams")

    data_params = sgqlc.types.Field(sgqlc.types.non_null(DataParams), graphql_name="dataParams")

    collector_name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="collectorName")


class JobSchema(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ("executor_method_schema", "model_methods_schema")
    executor_method_schema = sgqlc.types.Field(sgqlc.types.non_null("MethodSchema"), graphql_name="executorMethodSchema")

    model_methods_schema = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null("MethodSchema"))), graphql_name="modelMethodsSchema"
    )


class MethodParams(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ("method_name", "method_params")
    method_name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="methodName")

    method_params = sgqlc.types.Field(sgqlc.types.non_null(JSON), graphql_name="methodParams")


class MethodSchema(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ("schema_name", "json_schema")
    schema_name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="schemaName")

    json_schema = sgqlc.types.Field(sgqlc.types.non_null(JSON), graphql_name="jsonSchema")


class Metric(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ("key", "value", "step", "timestamp")
    key = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="key")

    value = sgqlc.types.Field(sgqlc.types.non_null(Float), graphql_name="value")

    step = sgqlc.types.Field(Int, graphql_name="step")

    timestamp = sgqlc.types.Field(sgqlc.types.non_null(LONG), graphql_name="timestamp")


class Model(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = (
        "name",
        "description",
        "owner_id",
        "tags",
        "creation_timestamp",
        "last_updated_timestamp",
        "owner",
        "latest_model_version",
        "init_model_version",
        "list_model_version",
        "pagination_model_version",
        "best_model_version",
    )
    name = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name="name")

    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="description")

    owner_id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name="ownerId")

    tags = sgqlc.types.Field(sgqlc.types.non_null(JSON), graphql_name="tags")

    creation_timestamp = sgqlc.types.Field(sgqlc.types.non_null(LONG), graphql_name="creationTimestamp")

    last_updated_timestamp = sgqlc.types.Field(LONG, graphql_name="lastUpdatedTimestamp")

    owner = sgqlc.types.Field(sgqlc.types.non_null("User"), graphql_name="owner")

    latest_model_version = sgqlc.types.Field(sgqlc.types.non_null("ModelVersion"), graphql_name="latestModelVersion")

    init_model_version = sgqlc.types.Field("ModelVersion", graphql_name="initModelVersion")

    list_model_version = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null("ModelVersion"))), graphql_name="listModelVersion"
    )

    pagination_model_version = sgqlc.types.Field(
        sgqlc.types.non_null("ModelVersionPagination"),
        graphql_name="paginationModelVersion",
        args=sgqlc.types.ArgDict(
            (
                ("limit", sgqlc.types.Arg(Int, graphql_name="limit", default=None)),
                ("offset", sgqlc.types.Arg(Int, graphql_name="offset", default=None)),
            )
        ),
    )
    """Arguments:

    * `limit` (`Int`)None (default: `null`)
    * `offset` (`Int`)None (default: `null`)
    """

    best_model_version = sgqlc.types.Field(
        sgqlc.types.non_null("ModelVersion"),
        graphql_name="bestModelVersion",
        args=sgqlc.types.ArgDict(
            (
                ("metric", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="metric", default=None)),
                ("optimal_min", sgqlc.types.Arg(Boolean, graphql_name="optimalMin", default=False)),
            )
        ),
    )
    """Arguments:

    * `metric` (`String!`)None
    * `optimal_min` (`Boolean`)None (default: `false`)
    """


class ModelPagination(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ("list_model", "total")
    list_model = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(Model))), graphql_name="listModel")

    total = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name="total")


class ModelVersion(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = (
        "name",
        "version",
        "source_path",
        "run_id",
        "root_run_id",
        "source_run_id",
        "source_model_name",
        "source_model_version",
        "default_experiment_name",
        "tags",
        "eval_run_ids",
        "description",
        "status",
        "status_message",
        "creation_timestamp",
        "last_updated_timestamp",
        "upload_model_type",
        "model_methods_schema",
        "docker_image_tag",
        "list_deployed_jobs",
        "pagination_deployed_jobs",
        "list_eval_run",
        "pagination_eval_run",
        "run",
        "source_run",
        "root_run",
        "model",
        "job_json_schema",
        "available_executor_versions",
        "available_dataset_loaders",
        "available_collectors",
        "list_next_graph_nodes",
        "list_next_model_version",
        "list_new_model_from_version",
        "list_new_version_from_version",
        "pagination_next_model_version",
        "pagination_new_model_from_version",
        "pagination_new_version_from_version",
        "root_model_version",
        "list_lineage_model_version",
        "pagination_lineage_model_version",
    )
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="name")

    version = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name="version")

    source_path = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="sourcePath")

    run_id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name="runId")

    root_run_id = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="rootRunId")

    source_run_id = sgqlc.types.Field(String, graphql_name="sourceRunId")

    source_model_name = sgqlc.types.Field(String, graphql_name="sourceModelName")

    source_model_version = sgqlc.types.Field("ModelVersion", graphql_name="sourceModelVersion")

    default_experiment_name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="defaultExperimentName")

    tags = sgqlc.types.Field(sgqlc.types.non_null(JSON), graphql_name="tags")

    eval_run_ids = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(String))), graphql_name="evalRunIds")

    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="description")

    status = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="status")

    status_message = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="statusMessage")

    creation_timestamp = sgqlc.types.Field(sgqlc.types.non_null(LONG), graphql_name="creationTimestamp")

    last_updated_timestamp = sgqlc.types.Field(LONG, graphql_name="lastUpdatedTimestamp")

    upload_model_type = sgqlc.types.Field(sgqlc.types.non_null(UploadModelType), graphql_name="uploadModelType")

    model_methods_schema = sgqlc.types.Field(sgqlc.types.non_null(JSON), graphql_name="modelMethodsSchema")

    docker_image_tag = sgqlc.types.Field(String, graphql_name="dockerImageTag")

    list_deployed_jobs = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(Job))), graphql_name="listDeployedJobs"
    )

    pagination_deployed_jobs = sgqlc.types.Field(
        sgqlc.types.non_null(JobPagination),
        graphql_name="paginationDeployedJobs",
        args=sgqlc.types.ArgDict(
            (
                ("limit", sgqlc.types.Arg(Int, graphql_name="limit", default=None)),
                ("offset", sgqlc.types.Arg(Int, graphql_name="offset", default=None)),
            )
        ),
    )
    """Arguments:

    * `limit` (`Int`)None (default: `null`)
    * `offset` (`Int`)None (default: `null`)
    """

    list_eval_run = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null("Run"))), graphql_name="listEvalRun")

    pagination_eval_run = sgqlc.types.Field(
        sgqlc.types.non_null("RunPagination"),
        graphql_name="paginationEvalRun",
        args=sgqlc.types.ArgDict(
            (
                ("limit", sgqlc.types.Arg(Int, graphql_name="limit", default=None)),
                ("offset", sgqlc.types.Arg(Int, graphql_name="offset", default=None)),
            )
        ),
    )
    """Arguments:

    * `limit` (`Int`)None (default: `null`)
    * `offset` (`Int`)None (default: `null`)
    """

    run = sgqlc.types.Field(sgqlc.types.non_null("Run"), graphql_name="run")

    source_run = sgqlc.types.Field("Run", graphql_name="sourceRun")

    root_run = sgqlc.types.Field(sgqlc.types.non_null("Run"), graphql_name="rootRun")

    model = sgqlc.types.Field(sgqlc.types.non_null(Model), graphql_name="model")

    job_json_schema = sgqlc.types.Field(
        sgqlc.types.non_null(JobSchema),
        graphql_name="jobJsonSchema",
        args=sgqlc.types.ArgDict(
            (
                ("executor_name", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="executorName", default=None)),
                ("executor_version", sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name="executorVersion", default=None)),
            )
        ),
    )
    """Arguments:

    * `executor_name` (`String!`)None
    * `executor_version` (`Int!`)None
    """

    available_executor_versions = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(ExecutorVersion))), graphql_name="availableExecutorVersions"
    )

    available_dataset_loaders = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(DatasetLoader))), graphql_name="availableDatasetLoaders"
    )

    available_collectors = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(String))), graphql_name="availableCollectors"
    )

    list_next_graph_nodes = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(GraphNode))), graphql_name="listNextGraphNodes"
    )

    list_next_model_version = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null("ModelVersion"))), graphql_name="listNextModelVersion"
    )

    list_new_model_from_version = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null("ModelVersion"))), graphql_name="listNewModelFromVersion"
    )

    list_new_version_from_version = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null("ModelVersion"))), graphql_name="listNewVersionFromVersion"
    )

    pagination_next_model_version = sgqlc.types.Field(
        sgqlc.types.non_null("ModelVersionPagination"),
        graphql_name="paginationNextModelVersion",
        args=sgqlc.types.ArgDict(
            (
                ("limit", sgqlc.types.Arg(Int, graphql_name="limit", default=None)),
                ("offset", sgqlc.types.Arg(Int, graphql_name="offset", default=None)),
            )
        ),
    )
    """Arguments:

    * `limit` (`Int`)None (default: `null`)
    * `offset` (`Int`)None (default: `null`)
    """

    pagination_new_model_from_version = sgqlc.types.Field(
        sgqlc.types.non_null("ModelVersionPagination"),
        graphql_name="paginationNewModelFromVersion",
        args=sgqlc.types.ArgDict(
            (
                ("limit", sgqlc.types.Arg(Int, graphql_name="limit", default=None)),
                ("offset", sgqlc.types.Arg(Int, graphql_name="offset", default=None)),
            )
        ),
    )
    """Arguments:

    * `limit` (`Int`)None (default: `null`)
    * `offset` (`Int`)None (default: `null`)
    """

    pagination_new_version_from_version = sgqlc.types.Field(
        sgqlc.types.non_null("ModelVersionPagination"),
        graphql_name="paginationNewVersionFromVersion",
        args=sgqlc.types.ArgDict(
            (
                ("limit", sgqlc.types.Arg(Int, graphql_name="limit", default=None)),
                ("offset", sgqlc.types.Arg(Int, graphql_name="offset", default=None)),
            )
        ),
    )
    """Arguments:

    * `limit` (`Int`)None (default: `null`)
    * `offset` (`Int`)None (default: `null`)
    """

    root_model_version = sgqlc.types.Field(sgqlc.types.non_null("ModelVersion"), graphql_name="rootModelVersion")

    list_lineage_model_version = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null("ModelVersion"))), graphql_name="listLineageModelVersion"
    )

    pagination_lineage_model_version = sgqlc.types.Field(
        sgqlc.types.non_null("ModelVersionPagination"),
        graphql_name="paginationLineageModelVersion",
        args=sgqlc.types.ArgDict(
            (
                ("limit", sgqlc.types.Arg(Int, graphql_name="limit", default=None)),
                ("offset", sgqlc.types.Arg(Int, graphql_name="offset", default=None)),
            )
        ),
    )
    """Arguments:

    * `limit` (`Int`)None (default: `null`)
    * `offset` (`Int`)None (default: `null`)
    """


class ModelVersionPagination(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ("list_model_version", "total")
    list_model_version = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(ModelVersion))), graphql_name="listModelVersion"
    )

    total = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name="total")


class Mutation(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = (
        "rename_experiment",
        "add_ml_job",
        "delete_model",
        "set_model_tag",
        "delete_model_tag",
        "set_model_description",
        "delete_model_version_from_name_version",
        "set_model_version_tag",
        "delete_model_version_tag",
        "set_model_version_description",
        "delete_dataset_loader",
        "set_dataset_loader_tag",
        "delete_dataset_loader_tag",
        "set_dataset_loader_description",
        "delete_dataset_loader_version_from_name_version",
        "set_dataset_loader_version_tag",
        "delete_dataset_loader_version_tag",
        "set_dataset_loader_version_description",
        "delete_executor",
        "set_executor_tag",
        "delete_executor_tag",
        "set_executor_description",
        "delete_executor_version_from_name_version",
        "set_executor_version_tag",
        "delete_executor_version_tag",
        "set_executor_version_description",
    )
    rename_experiment = sgqlc.types.Field(
        sgqlc.types.non_null(Experiment),
        graphql_name="renameExperiment",
        args=sgqlc.types.ArgDict(
            (
                ("name", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="name", default=None)),
                ("new_name", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="newName", default=None)),
            )
        ),
    )
    """Arguments:

    * `name` (`String!`)None
    * `new_name` (`String!`)None
    """

    add_ml_job = sgqlc.types.Field(
        sgqlc.types.non_null(Job),
        graphql_name="addMlJob",
        args=sgqlc.types.ArgDict(
            (
                ("job_executor_name", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="jobExecutorName", default=None)),
                ("model_name", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="modelName", default=None)),
                ("dataset_loader_name", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="datasetLoaderName", default=None)),
                (
                    "executor_model_params",
                    sgqlc.types.Arg(sgqlc.types.non_null(ExecutorModelParamsInput), graphql_name="executorModelParams", default=None),
                ),
                ("data_params", sgqlc.types.Arg(sgqlc.types.non_null(DataParamsInput), graphql_name="dataParams", default=None)),
                ("collector_name", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="collectorName", default=None)),
                ("choice_criteria", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="choiceCriteria", default="latest")),
                ("dataset_loader_version", sgqlc.types.Arg(Int, graphql_name="datasetLoaderVersion", default=None)),
                ("job_executor_version", sgqlc.types.Arg(Int, graphql_name="jobExecutorVersion", default=None)),
                ("model_version", sgqlc.types.Arg(Int, graphql_name="modelVersion", default=None)),
                ("new_model_name", sgqlc.types.Arg(String, graphql_name="newModelName", default=None)),
                ("experiment_name", sgqlc.types.Arg(String, graphql_name="experimentName", default=None)),
                ("cron_expression", sgqlc.types.Arg(String, graphql_name="cronExpression", default=None)),
                ("periodic_type", sgqlc.types.Arg(PeriodicType, graphql_name="periodicType", default="ONCE")),
                ("metric_name", sgqlc.types.Arg(String, graphql_name="metricName", default=None)),
                ("optimal_min", sgqlc.types.Arg(sgqlc.types.non_null(Boolean), graphql_name="optimalMin", default=False)),
                ("force_rebuild", sgqlc.types.Arg(Boolean, graphql_name="forceRebuild", default=False)),
                ("gpu", sgqlc.types.Arg(Boolean, graphql_name="gpu", default=False)),
            )
        ),
    )
    """Arguments:

    * `job_executor_name` (`String!`)None
    * `model_name` (`String!`)None
    * `dataset_loader_name` (`String!`)None
    * `executor_model_params` (`ExecutorModelParamsInput!`)None
    * `data_params` (`DataParamsInput!`)None
    * `collector_name` (`String!`)None
    * `choice_criteria` (`String!`)None (default: `"latest"`)
    * `dataset_loader_version` (`Int`)None (default: `null`)
    * `job_executor_version` (`Int`)None (default: `null`)
    * `model_version` (`Int`)None (default: `null`)
    * `new_model_name` (`String`)None (default: `null`)
    * `experiment_name` (`String`)None (default: `null`)
    * `cron_expression` (`String`)None (default: `null`)
    * `periodic_type` (`PeriodicType`)None (default: `ONCE`)
    * `metric_name` (`String`)None (default: `null`)
    * `optimal_min` (`Boolean!`)None (default: `false`)
    * `force_rebuild` (`Boolean`)None (default: `false`)
    * `gpu` (`Boolean`)None (default: `false`)
    """

    delete_model = sgqlc.types.Field(
        sgqlc.types.non_null(Boolean),
        graphql_name="deleteModel",
        args=sgqlc.types.ArgDict((("name", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="name", default=None)),)),
    )
    """Arguments:

    * `name` (`String!`)None
    """

    set_model_tag = sgqlc.types.Field(
        sgqlc.types.non_null(Model),
        graphql_name="setModelTag",
        args=sgqlc.types.ArgDict(
            (
                ("name", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="name", default=None)),
                ("key", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="key", default=None)),
                ("value", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="value", default=None)),
            )
        ),
    )
    """Arguments:

    * `name` (`String!`)None
    * `key` (`String!`)None
    * `value` (`String!`)None
    """

    delete_model_tag = sgqlc.types.Field(
        sgqlc.types.non_null(Model),
        graphql_name="deleteModelTag",
        args=sgqlc.types.ArgDict(
            (
                ("name", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="name", default=None)),
                ("key", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="key", default=None)),
            )
        ),
    )
    """Arguments:

    * `name` (`String!`)None
    * `key` (`String!`)None
    """

    set_model_description = sgqlc.types.Field(
        sgqlc.types.non_null(Model),
        graphql_name="setModelDescription",
        args=sgqlc.types.ArgDict(
            (
                ("name", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="name", default=None)),
                ("description", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="description", default=None)),
            )
        ),
    )
    """Arguments:

    * `name` (`String!`)None
    * `description` (`String!`)None
    """

    delete_model_version_from_name_version = sgqlc.types.Field(
        sgqlc.types.non_null(Boolean),
        graphql_name="deleteModelVersionFromNameVersion",
        args=sgqlc.types.ArgDict(
            (("model_version", sgqlc.types.Arg(sgqlc.types.non_null(ObjectVersionInput), graphql_name="modelVersion", default=None)),)
        ),
    )
    """Arguments:

    * `model_version` (`ObjectVersionInput!`)None
    """

    set_model_version_tag = sgqlc.types.Field(
        sgqlc.types.non_null(ModelVersion),
        graphql_name="setModelVersionTag",
        args=sgqlc.types.ArgDict(
            (
                ("model_version", sgqlc.types.Arg(sgqlc.types.non_null(ObjectVersionInput), graphql_name="modelVersion", default=None)),
                ("key", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="key", default=None)),
                ("value", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="value", default=None)),
            )
        ),
    )
    """Arguments:

    * `model_version` (`ObjectVersionInput!`)None
    * `key` (`String!`)None
    * `value` (`String!`)None
    """

    delete_model_version_tag = sgqlc.types.Field(
        sgqlc.types.non_null(ModelVersion),
        graphql_name="deleteModelVersionTag",
        args=sgqlc.types.ArgDict(
            (
                ("model_version", sgqlc.types.Arg(sgqlc.types.non_null(ObjectVersionInput), graphql_name="modelVersion", default=None)),
                ("key", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="key", default=None)),
            )
        ),
    )
    """Arguments:

    * `model_version` (`ObjectVersionInput!`)None
    * `key` (`String!`)None
    """

    set_model_version_description = sgqlc.types.Field(
        sgqlc.types.non_null(ModelVersion),
        graphql_name="setModelVersionDescription",
        args=sgqlc.types.ArgDict(
            (
                ("model_version", sgqlc.types.Arg(sgqlc.types.non_null(ObjectVersionInput), graphql_name="modelVersion", default=None)),
                ("description", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="description", default=None)),
            )
        ),
    )
    """Arguments:

    * `model_version` (`ObjectVersionInput!`)None
    * `description` (`String!`)None
    """

    delete_dataset_loader = sgqlc.types.Field(
        sgqlc.types.non_null(Boolean),
        graphql_name="deleteDatasetLoader",
        args=sgqlc.types.ArgDict((("name", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="name", default=None)),)),
    )
    """Arguments:

    * `name` (`String!`)None
    """

    set_dataset_loader_tag = sgqlc.types.Field(
        sgqlc.types.non_null(DatasetLoader),
        graphql_name="setDatasetLoaderTag",
        args=sgqlc.types.ArgDict(
            (
                ("name", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="name", default=None)),
                ("key", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="key", default=None)),
                ("value", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="value", default=None)),
            )
        ),
    )
    """Arguments:

    * `name` (`String!`)None
    * `key` (`String!`)None
    * `value` (`String!`)None
    """

    delete_dataset_loader_tag = sgqlc.types.Field(
        sgqlc.types.non_null(DatasetLoader),
        graphql_name="deleteDatasetLoaderTag",
        args=sgqlc.types.ArgDict(
            (
                ("name", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="name", default=None)),
                ("key", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="key", default=None)),
            )
        ),
    )
    """Arguments:

    * `name` (`String!`)None
    * `key` (`String!`)None
    """

    set_dataset_loader_description = sgqlc.types.Field(
        sgqlc.types.non_null(DatasetLoader),
        graphql_name="setDatasetLoaderDescription",
        args=sgqlc.types.ArgDict(
            (
                ("name", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="name", default=None)),
                ("description", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="description", default=None)),
            )
        ),
    )
    """Arguments:

    * `name` (`String!`)None
    * `description` (`String!`)None
    """

    delete_dataset_loader_version_from_name_version = sgqlc.types.Field(
        Boolean,
        graphql_name="deleteDatasetLoaderVersionFromNameVersion",
        args=sgqlc.types.ArgDict(
            (
                (
                    "dataset_loader_version",
                    sgqlc.types.Arg(sgqlc.types.non_null(ObjectVersionInput), graphql_name="datasetLoaderVersion", default=None),
                ),
            )
        ),
    )
    """Arguments:

    * `dataset_loader_version` (`ObjectVersionInput!`)None
    """

    set_dataset_loader_version_tag = sgqlc.types.Field(
        sgqlc.types.non_null(DatasetLoaderVersion),
        graphql_name="setDatasetLoaderVersionTag",
        args=sgqlc.types.ArgDict(
            (
                (
                    "dataset_loader_version",
                    sgqlc.types.Arg(sgqlc.types.non_null(ObjectVersionInput), graphql_name="datasetLoaderVersion", default=None),
                ),
                ("key", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="key", default=None)),
                ("value", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="value", default=None)),
            )
        ),
    )
    """Arguments:

    * `dataset_loader_version` (`ObjectVersionInput!`)None
    * `key` (`String!`)None
    * `value` (`String!`)None
    """

    delete_dataset_loader_version_tag = sgqlc.types.Field(
        sgqlc.types.non_null(DatasetLoaderVersion),
        graphql_name="deleteDatasetLoaderVersionTag",
        args=sgqlc.types.ArgDict(
            (
                (
                    "dataset_loader_version",
                    sgqlc.types.Arg(sgqlc.types.non_null(ObjectVersionInput), graphql_name="datasetLoaderVersion", default=None),
                ),
                ("key", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="key", default=None)),
            )
        ),
    )
    """Arguments:

    * `dataset_loader_version` (`ObjectVersionInput!`)None
    * `key` (`String!`)None
    """

    set_dataset_loader_version_description = sgqlc.types.Field(
        sgqlc.types.non_null(DatasetLoaderVersion),
        graphql_name="setDatasetLoaderVersionDescription",
        args=sgqlc.types.ArgDict(
            (
                (
                    "dataset_loader_version",
                    sgqlc.types.Arg(sgqlc.types.non_null(ObjectVersionInput), graphql_name="datasetLoaderVersion", default=None),
                ),
                ("description", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="description", default=None)),
            )
        ),
    )
    """Arguments:

    * `dataset_loader_version` (`ObjectVersionInput!`)None
    * `description` (`String!`)None
    """

    delete_executor = sgqlc.types.Field(
        sgqlc.types.non_null(Boolean),
        graphql_name="deleteExecutor",
        args=sgqlc.types.ArgDict((("name", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="name", default=None)),)),
    )
    """Arguments:

    * `name` (`String!`)None
    """

    set_executor_tag = sgqlc.types.Field(
        sgqlc.types.non_null(Executor),
        graphql_name="setExecutorTag",
        args=sgqlc.types.ArgDict(
            (
                ("name", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="name", default=None)),
                ("key", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="key", default=None)),
                ("value", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="value", default=None)),
            )
        ),
    )
    """Arguments:

    * `name` (`String!`)None
    * `key` (`String!`)None
    * `value` (`String!`)None
    """

    delete_executor_tag = sgqlc.types.Field(
        sgqlc.types.non_null(Executor),
        graphql_name="deleteExecutorTag",
        args=sgqlc.types.ArgDict(
            (
                ("name", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="name", default=None)),
                ("key", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="key", default=None)),
            )
        ),
    )
    """Arguments:

    * `name` (`String!`)None
    * `key` (`String!`)None
    """

    set_executor_description = sgqlc.types.Field(
        sgqlc.types.non_null(Executor),
        graphql_name="setExecutorDescription",
        args=sgqlc.types.ArgDict(
            (
                ("name", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="name", default=None)),
                ("description", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="description", default=None)),
            )
        ),
    )
    """Arguments:

    * `name` (`String!`)None
    * `description` (`String!`)None
    """

    delete_executor_version_from_name_version = sgqlc.types.Field(
        Boolean,
        graphql_name="deleteExecutorVersionFromNameVersion",
        args=sgqlc.types.ArgDict(
            (("executor_version", sgqlc.types.Arg(sgqlc.types.non_null(ObjectVersionInput), graphql_name="executorVersion", default=None)),)
        ),
    )
    """Arguments:

    * `executor_version` (`ObjectVersionInput!`)None
    """

    set_executor_version_tag = sgqlc.types.Field(
        sgqlc.types.non_null(ExecutorVersion),
        graphql_name="setExecutorVersionTag",
        args=sgqlc.types.ArgDict(
            (
                (
                    "executor_version",
                    sgqlc.types.Arg(sgqlc.types.non_null(ObjectVersionInput), graphql_name="executorVersion", default=None),
                ),
                ("key", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="key", default=None)),
                ("value", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="value", default=None)),
            )
        ),
    )
    """Arguments:

    * `executor_version` (`ObjectVersionInput!`)None
    * `key` (`String!`)None
    * `value` (`String!`)None
    """

    delete_executor_version_tag = sgqlc.types.Field(
        sgqlc.types.non_null(ExecutorVersion),
        graphql_name="deleteExecutorVersionTag",
        args=sgqlc.types.ArgDict(
            (
                (
                    "executor_version",
                    sgqlc.types.Arg(sgqlc.types.non_null(ObjectVersionInput), graphql_name="executorVersion", default=None),
                ),
                ("key", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="key", default=None)),
            )
        ),
    )
    """Arguments:

    * `executor_version` (`ObjectVersionInput!`)None
    * `key` (`String!`)None
    """

    set_executor_version_description = sgqlc.types.Field(
        sgqlc.types.non_null(ExecutorVersion),
        graphql_name="setExecutorVersionDescription",
        args=sgqlc.types.ArgDict(
            (
                (
                    "executor_version",
                    sgqlc.types.Arg(sgqlc.types.non_null(ObjectVersionInput), graphql_name="executorVersion", default=None),
                ),
                ("description", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="description", default=None)),
            )
        ),
    )
    """Arguments:

    * `executor_version` (`ObjectVersionInput!`)None
    * `description` (`String!`)None
    """


class Query(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = (
        "graph_node",
        "list_graph_node",
        "list_subtree_graph_node",
        "run_from_run_id",
        "search_runs",
        "list_experiment",
        "pagination_experiment",
        "experiment_from_name",
        "experiment_from_id",
        "job_from_name",
        "list_job",
        "pagination_job",
        "search_job",
        "model_from_name",
        "warning_delete_model_query",
        "list_model",
        "pagination_model",
        "model_version_from_run_id",
        "model_version_from_name_version",
        "list_initial_model_version",
        "pagination_initial_model_version",
        "list_dataset_loader",
        "pagination_dataset_loader",
        "dataset_loader_from_name",
        "dataset_loader_version_from_name_version",
        "dataset_loader_version_from_run_id",
        "executor_from_name",
        "list_executor",
        "pagination_executor",
        "executor_version_from_name_version",
        "executor_version_from_run_id",
        "list_initial_executor_version",
    )
    graph_node = sgqlc.types.Field(
        sgqlc.types.non_null(GraphNode),
        graphql_name="graphNode",
        args=sgqlc.types.ArgDict((("run_id", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="runId", default=None)),)),
    )
    """Arguments:

    * `run_id` (`String!`)None
    """

    list_graph_node = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(GraphNode))), graphql_name="listGraphNode"
    )

    list_subtree_graph_node = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(GraphNode))),
        graphql_name="listSubtreeGraphNode",
        args=sgqlc.types.ArgDict((("root_run_id", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="rootRunId", default=None)),)),
    )
    """Arguments:

    * `root_run_id` (`String!`)None
    """

    run_from_run_id = sgqlc.types.Field(
        sgqlc.types.non_null("Run"),
        graphql_name="runFromRunId",
        args=sgqlc.types.ArgDict((("run_id", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="runId", default=None)),)),
    )
    """Arguments:

    * `run_id` (`String!`)None
    """

    search_runs = sgqlc.types.Field(
        sgqlc.types.non_null("RunPagination"),
        graphql_name="searchRuns",
        args=sgqlc.types.ArgDict(
            (
                (
                    "experiment_ids",
                    sgqlc.types.Arg(sgqlc.types.list_of(sgqlc.types.non_null(String)), graphql_name="experimentIds", default=None),
                ),
                ("limit", sgqlc.types.Arg(Int, graphql_name="limit", default=None)),
                ("offset", sgqlc.types.Arg(Int, graphql_name="offset", default=None)),
                ("filter_string", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="filterString", default="")),
                ("run_view_type", sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name="runViewType", default=1)),
                ("order_by", sgqlc.types.Arg(sgqlc.types.list_of(sgqlc.types.non_null(String)), graphql_name="orderBy", default=None)),
                (
                    "experiment_names",
                    sgqlc.types.Arg(sgqlc.types.list_of(sgqlc.types.non_null(String)), graphql_name="experimentNames", default=None),
                ),
            )
        ),
    )
    """Arguments:

    * `experiment_ids` (`[String!]`)None (default: `null`)
    * `limit` (`Int`)None (default: `null`)
    * `offset` (`Int`)None (default: `null`)
    * `filter_string` (`String!`)None (default: `""`)
    * `run_view_type` (`Int!`)None (default: `1`)
    * `order_by` (`[String!]`)None (default: `null`)
    * `experiment_names` (`[String!]`)None (default: `null`)
    """

    list_experiment = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(Experiment))), graphql_name="listExperiment"
    )

    pagination_experiment = sgqlc.types.Field(
        sgqlc.types.non_null(ExperimentPagination),
        graphql_name="paginationExperiment",
        args=sgqlc.types.ArgDict(
            (
                ("limit", sgqlc.types.Arg(Int, graphql_name="limit", default=None)),
                ("offset", sgqlc.types.Arg(Int, graphql_name="offset", default=None)),
            )
        ),
    )
    """Arguments:

    * `limit` (`Int`)None (default: `null`)
    * `offset` (`Int`)None (default: `null`)
    """

    experiment_from_name = sgqlc.types.Field(
        sgqlc.types.non_null(Experiment),
        graphql_name="experimentFromName",
        args=sgqlc.types.ArgDict((("name", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="name", default=None)),)),
    )
    """Arguments:

    * `name` (`String!`)None
    """

    experiment_from_id = sgqlc.types.Field(
        sgqlc.types.non_null(Experiment),
        graphql_name="experimentFromId",
        args=sgqlc.types.ArgDict(
            (("experiment_id", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="experimentId", default=None)),)
        ),
    )
    """Arguments:

    * `experiment_id` (`String!`)None
    """

    job_from_name = sgqlc.types.Field(
        sgqlc.types.non_null(Job),
        graphql_name="jobFromName",
        args=sgqlc.types.ArgDict((("name", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="name", default=None)),)),
    )
    """Arguments:

    * `name` (`String!`)None
    """

    list_job = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(Job))), graphql_name="listJob")

    pagination_job = sgqlc.types.Field(
        sgqlc.types.non_null(JobPagination),
        graphql_name="paginationJob",
        args=sgqlc.types.ArgDict(
            (
                ("limit", sgqlc.types.Arg(Int, graphql_name="limit", default=None)),
                ("offset", sgqlc.types.Arg(Int, graphql_name="offset", default=None)),
            )
        ),
    )
    """Arguments:

    * `limit` (`Int`)None (default: `null`)
    * `offset` (`Int`)None (default: `null`)
    """

    search_job = sgqlc.types.Field(
        sgqlc.types.non_null(JobPagination),
        graphql_name="searchJob",
        args=sgqlc.types.ArgDict(
            (
                ("periodic_type", sgqlc.types.Arg(PeriodicType, graphql_name="periodicType", default=None)),
                ("status", sgqlc.types.Arg(JobStatus, graphql_name="status", default=None)),
                ("init_model_version", sgqlc.types.Arg(ObjectVersionInput, graphql_name="initModelVersion", default=None)),
                ("dataset_loader_version", sgqlc.types.Arg(ObjectVersionInput, graphql_name="datasetLoaderVersion", default=None)),
                ("executor_version", sgqlc.types.Arg(ObjectVersionInput, graphql_name="executorVersion", default=None)),
                ("start_interval", sgqlc.types.Arg(TimestampInterval, graphql_name="startInterval", default=None)),
                ("end_interval", sgqlc.types.Arg(TimestampInterval, graphql_name="endInterval", default=None)),
                ("limit", sgqlc.types.Arg(Int, graphql_name="limit", default=None)),
                ("offset", sgqlc.types.Arg(Int, graphql_name="offset", default=None)),
            )
        ),
    )
    """Arguments:

    * `periodic_type` (`PeriodicType`)None (default: `null`)
    * `status` (`JobStatus`)None (default: `null`)
    * `init_model_version` (`ObjectVersionInput`)None (default:
      `null`)
    * `dataset_loader_version` (`ObjectVersionInput`)None (default:
      `null`)
    * `executor_version` (`ObjectVersionInput`)None (default: `null`)
    * `start_interval` (`TimestampInterval`)None (default: `null`)
    * `end_interval` (`TimestampInterval`)None (default: `null`)
    * `limit` (`Int`)None (default: `null`)
    * `offset` (`Int`)None (default: `null`)
    """

    model_from_name = sgqlc.types.Field(
        sgqlc.types.non_null(Model),
        graphql_name="modelFromName",
        args=sgqlc.types.ArgDict((("name", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="name", default=None)),)),
    )
    """Arguments:

    * `name` (`String!`)None
    """

    warning_delete_model_query = sgqlc.types.Field(
        sgqlc.types.non_null("WarningDeleteModel"),
        graphql_name="warningDeleteModelQuery",
        args=sgqlc.types.ArgDict((("name", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="name", default=None)),)),
    )
    """Arguments:

    * `name` (`String!`)None
    """

    list_model = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(Model))), graphql_name="listModel")

    pagination_model = sgqlc.types.Field(
        sgqlc.types.non_null(ModelPagination),
        graphql_name="paginationModel",
        args=sgqlc.types.ArgDict(
            (
                ("limit", sgqlc.types.Arg(Int, graphql_name="limit", default=None)),
                ("offset", sgqlc.types.Arg(Int, graphql_name="offset", default=None)),
            )
        ),
    )
    """Arguments:

    * `limit` (`Int`)None (default: `null`)
    * `offset` (`Int`)None (default: `null`)
    """

    model_version_from_run_id = sgqlc.types.Field(
        sgqlc.types.non_null(ModelVersion),
        graphql_name="modelVersionFromRunId",
        args=sgqlc.types.ArgDict((("run_id", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="runId", default=None)),)),
    )
    """Arguments:

    * `run_id` (`String!`)None
    """

    model_version_from_name_version = sgqlc.types.Field(
        sgqlc.types.non_null(ModelVersion),
        graphql_name="modelVersionFromNameVersion",
        args=sgqlc.types.ArgDict(
            (("model_version", sgqlc.types.Arg(sgqlc.types.non_null(ObjectVersionInput), graphql_name="modelVersion", default=None)),)
        ),
    )
    """Arguments:

    * `model_version` (`ObjectVersionInput!`)None
    """

    list_initial_model_version = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(ModelVersion))), graphql_name="listInitialModelVersion"
    )

    pagination_initial_model_version = sgqlc.types.Field(
        sgqlc.types.non_null(ModelVersionPagination),
        graphql_name="paginationInitialModelVersion",
        args=sgqlc.types.ArgDict(
            (
                ("limit", sgqlc.types.Arg(Int, graphql_name="limit", default=None)),
                ("offset", sgqlc.types.Arg(Int, graphql_name="offset", default=None)),
            )
        ),
    )
    """Arguments:

    * `limit` (`Int`)None (default: `null`)
    * `offset` (`Int`)None (default: `null`)
    """

    list_dataset_loader = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(DatasetLoader))), graphql_name="listDatasetLoader"
    )

    pagination_dataset_loader = sgqlc.types.Field(
        sgqlc.types.non_null(DatasetLoaderPagination),
        graphql_name="paginationDatasetLoader",
        args=sgqlc.types.ArgDict(
            (
                ("limit", sgqlc.types.Arg(Int, graphql_name="limit", default=None)),
                ("offset", sgqlc.types.Arg(Int, graphql_name="offset", default=None)),
            )
        ),
    )
    """Arguments:

    * `limit` (`Int`)None (default: `null`)
    * `offset` (`Int`)None (default: `null`)
    """

    dataset_loader_from_name = sgqlc.types.Field(
        sgqlc.types.non_null(DatasetLoader),
        graphql_name="datasetLoaderFromName",
        args=sgqlc.types.ArgDict((("name", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="name", default=None)),)),
    )
    """Arguments:

    * `name` (`String!`)None
    """

    dataset_loader_version_from_name_version = sgqlc.types.Field(
        sgqlc.types.non_null(DatasetLoaderVersion),
        graphql_name="datasetLoaderVersionFromNameVersion",
        args=sgqlc.types.ArgDict(
            (
                (
                    "dataset_loader_version",
                    sgqlc.types.Arg(sgqlc.types.non_null(ObjectVersionInput), graphql_name="datasetLoaderVersion", default=None),
                ),
            )
        ),
    )
    """Arguments:

    * `dataset_loader_version` (`ObjectVersionInput!`)None
    """

    dataset_loader_version_from_run_id = sgqlc.types.Field(
        sgqlc.types.non_null(DatasetLoaderVersion),
        graphql_name="datasetLoaderVersionFromRunId",
        args=sgqlc.types.ArgDict((("run_id", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="runId", default=None)),)),
    )
    """Arguments:

    * `run_id` (`String!`)None
    """

    executor_from_name = sgqlc.types.Field(
        sgqlc.types.non_null(Executor),
        graphql_name="executorFromName",
        args=sgqlc.types.ArgDict((("name", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="name", default=None)),)),
    )
    """Arguments:

    * `name` (`String!`)None
    """

    list_executor = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(Executor))), graphql_name="listExecutor"
    )

    pagination_executor = sgqlc.types.Field(
        sgqlc.types.non_null(ExecutorPagination),
        graphql_name="paginationExecutor",
        args=sgqlc.types.ArgDict(
            (
                ("limit", sgqlc.types.Arg(Int, graphql_name="limit", default=None)),
                ("offset", sgqlc.types.Arg(Int, graphql_name="offset", default=None)),
            )
        ),
    )
    """Arguments:

    * `limit` (`Int`)None (default: `null`)
    * `offset` (`Int`)None (default: `null`)
    """

    executor_version_from_name_version = sgqlc.types.Field(
        sgqlc.types.non_null(ExecutorVersion),
        graphql_name="executorVersionFromNameVersion",
        args=sgqlc.types.ArgDict(
            (("executor_version", sgqlc.types.Arg(sgqlc.types.non_null(ObjectVersionInput), graphql_name="executorVersion", default=None)),)
        ),
    )
    """Arguments:

    * `executor_version` (`ObjectVersionInput!`)None
    """

    executor_version_from_run_id = sgqlc.types.Field(
        sgqlc.types.non_null(ExecutorVersion),
        graphql_name="executorVersionFromRunId",
        args=sgqlc.types.ArgDict((("run_id", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="runId", default=None)),)),
    )
    """Arguments:

    * `run_id` (`String!`)None
    """

    list_initial_executor_version = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(ExecutorVersion))), graphql_name="listInitialExecutorVersion"
    )


class Run(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = (
        "run_id",
        "artifact_uri",
        "status",
        "latest_metrics",
        "experiment_id",
        "params",
        "start_time",
        "end_time",
        "lifecycle_stage",
        "parent_job",
        "list_artifacts",
        "experiment",
        "tags",
        "metric_history",
    )
    run_id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name="runId")

    artifact_uri = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="artifactUri")

    status = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="status")

    latest_metrics = sgqlc.types.Field(sgqlc.types.non_null(JSON), graphql_name="latestMetrics")

    experiment_id = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="experimentId")

    params = sgqlc.types.Field(sgqlc.types.non_null(JSON), graphql_name="params")

    start_time = sgqlc.types.Field(sgqlc.types.non_null(LONG), graphql_name="startTime")

    end_time = sgqlc.types.Field(sgqlc.types.non_null(LONG), graphql_name="endTime")

    lifecycle_stage = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="lifecycleStage")

    parent_job = sgqlc.types.Field(Job, graphql_name="parentJob")

    list_artifacts = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(String))), graphql_name="listArtifacts"
    )

    experiment = sgqlc.types.Field(sgqlc.types.non_null(Experiment), graphql_name="experiment")

    tags = sgqlc.types.Field(sgqlc.types.non_null(JSON), graphql_name="tags")

    metric_history = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(Metric))),
        graphql_name="metricHistory",
        args=sgqlc.types.ArgDict((("metric", sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name="metric", default=None)),)),
    )
    """Arguments:

    * `metric` (`String!`)None
    """


class RunPagination(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ("list_run", "total")
    list_run = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(Run))), graphql_name="listRun")

    total = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name="total")


class User(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ("id",)
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name="id")


class WarningDeleteModel(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ("delete_possible", "list_model_version")
    delete_possible = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name="deletePossible")

    list_model_version = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(ModelVersion))), graphql_name="listModelVersion"
    )


########################################################################
# Unions
########################################################################

########################################################################
# Schema Entry Points
########################################################################
schema.query_type = Query
schema.mutation_type = Mutation
schema.subscription_type = None



"""Common variables."""
import os
import posixpath

server_url = None

active_run_stack = []
active_experiment_name = None

EXPERIMENT_NAME_FOR_EXECUTOR = "executors"
EXPERIMENT_NAME_FOR_DATASET_LOADER = "dataset_loaders"


def _get_server_ml_api() -> str:
    """Get server '/mlflow' endpoint URL."""
    return posixpath.join(_get_server_url(), "mlflow")


def _get_server_url() -> str:
    """
    Get server URL.

    If you set the URL using 'mlmanagement.set_server_url' function,
    it takes precedence over the URL from the environment variable 'server_url'
    """
    return os.environ.get("server_url", "http://mlmanagement.ru") if not server_url else server_url



"""SDK for client library."""
import json
import posixpath
from typing import Any, Dict, List, Optional

import pandas as pd
from jsf import JSF
from ML_management.mlmanagement import get_server_url
from ML_management.mlmanagement.session import AuthSession
from ML_management.models.model_type_to_methods_map import ModelMethodName
from ML_management.registry.registry_manager import RegistryManager
from ML_management.sdk import schema
from ML_management.sdk.schema import Job, ModelVersion
from sgqlc.operation import Operation


def _to_datetime(df: pd.DataFrame, column_names: List[str]) -> pd.DataFrame:
    """
    Convert df's columns to datetime.

    Parameters
    ----------
    df: pd.DataFrame
        pd.DataFrame in which the columns will be converted.
    column_names: List[str]
        Сolumn names to be converted.

    Returns
    -------
    pd.DataFrame
        Pandas dataframe with converted columns.
    """
    for column_name in column_names:
        df[column_name] = pd.to_datetime(df[column_name], unit="s")

    return df


def send_graphql_request(op: Operation, json_response: bool = True) -> Any:
    """Send request to server and process the response."""
    json_data = AuthSession().sgqlc_request(op)

    if "data" not in json_data or json_data["data"] is None:
        raise Exception(json_data["errors"][0]["message"])

    if json_response:
        return json_data["data"]
    else:
        return op + json_data


def list_model() -> pd.DataFrame:
    """
    List available models.

    Returns
    -------
    pd.DataFrame
        Pandas dataframe with list of available models.
    """
    op = Operation(schema.Query)
    op.list_model.name()
    op.list_model.description()
    op.list_model.creation_timestamp()
    op.list_model.last_updated_timestamp()
    json_data = send_graphql_request(op)
    df = pd.DataFrame.from_dict(json_data["listModel"])
    if not df.empty:
        df = _to_datetime(df, ["creationTimestamp", "lastUpdatedTimestamp"])
    return df


def list_dataset_loader() -> pd.DataFrame:
    """
    List available dataset_loaders.

    Returns
    -------
    pd.DataFrame
        Pandas dataframe with list of available dataset_loaders.
    """
    op = Operation(schema.Query)
    op.list_dataset_loader.name()
    op.list_dataset_loader.description()
    op.list_dataset_loader.creation_timestamp()
    op.list_dataset_loader.last_updated_timestamp()
    json_data = send_graphql_request(op)
    df = pd.DataFrame.from_dict(json_data["listDatasetLoader"])
    if not df.empty:
        df = _to_datetime(df, ["creationTimestamp", "lastUpdatedTimestamp"])
    return df


def list_executor() -> pd.DataFrame:
    """
    List available executors.

    Returns
    -------
    pd.DataFrame
        Pandas dataframe with list of available executors.
    """
    op = Operation(schema.Query)
    op.list_executor.name()
    op.list_executor.description()
    op.list_executor.creation_timestamp()
    op.list_executor.last_updated_timestamp()
    json_data = send_graphql_request(op)
    df = pd.DataFrame.from_dict(json_data["listExecutor"])
    if not df.empty:
        df = _to_datetime(df, ["creationTimestamp", "lastUpdatedTimestamp"])
    return df


def add_ml_job(
    job_executor_name: str,
    executor_params: dict,
    dataset_loader_name: str,
    dataset_params: dict,
    model_name: str,
    model_params: List[Dict[str, dict]],
    collector_params: dict,
    gpu: bool = False,
    job_executor_version: Optional[int] = None,
    dataset_loader_version: Optional[int] = None,
    model_version: Optional[int] = None,
    new_model_name: Optional[str] = None,
    experiment_name: str = "Default",
    collector_name: str = "s3",
    choice_criteria: str = "latest",
    cron_expression: Optional[str] = None,
    periodic_type: str = "ONCE",
    metric_name: Optional[str] = None,
    optimal_min: bool = False,
    force_rebuild: bool = False,
) -> Job:
    """
    Create execution job.

    Parameters
    ----------
    job_executor_name: str
        Name of the executor that will execute the job.
    executor_params: Dict[str, ...]
        Dictionary of executor parameters. Example: {'executor_param1': list,
                                                     'executor_param2': int,
                                                     'executor_param3': dict,
                                                     ...}
    dataset_loader_name: str
        Name of the DatasetLoader that the model will use.
    dataset_params: dict
        Dictionary of DatasetLoader parameters.
    model_name: str
        Name of the model to interact with.
    model_params: List[Dict[str, dict]]
        List of dictionaries with parameters of model methods. Example: [{"method_one": {key1: value1, key2: value2}},
                                                                         {"method_two": {key1: value1, key2: value2}}]
    collector_params: dict
        Dictionary of collector parameters. Example: {'collector_param1': list,
                                                      'collector_param2': int,
                                                      'collector_param3': dict,
                                                     ...}
    gpu: bool = False
        Whether to use GPU for this job or not. Default: False
    job_executor_version: Optional[int] = None
        Version of the executor that will execute the job. Default: None, "latest" version is used.
    dataset_loader_version: Optional[int] = None
        Version of the DatasetLoader that the model will interact with. Default: None, "latest" version is used.
    model_version: Optional[int] = None
        Version of the model to interact with. Default: None, "latest" version is used.
    new_model_name: Optional[str] = None
        Name of the model to save in case new model is to be created as a result of job execution (regulated by executor.upload_model_mode).
        Default: None.
    experiment_name: str = "Default"
        Name of the experiment. Default: "Default"
    collector_name: {"s3", }
        Name of the collector to interact with. Default: "s3"
    choice_criteria: {"latest", "initial", "best"}
        Criteria to choose the model to interact with. Required if model_version is not specified. Default: "latest"
        If "best": according to the optimal_min value, minimum (or maximum) value for the selected metric_name is taken
    cron_expression: str = None
        Cron expression for periodic or deferred jobs. Default: None
    periodic_type: {"ONCE", "PERIODIC"}
        Frequency of the task. Default: "ONCE"
    metric_name: str = None
        Name of the metric(must be logged in) for choice_criteria. Default: None
    optimal_min: bool = False
        Whether to take minimum or maximum value for the selected choice_criteria and metric_name. Default: False
    force_rebuild: bool = False
        Whether to rebuild the image of jobs, since model images are cached to speed up the build process. Default: False

    Returns
    -------
    Job
        Instance of the Job class.
    """
    op = Operation(schema.Query)

    if dataset_loader_version is None:
        dataset_loader_version = int(RegistryManager().get_latest_dataset_loader_version(name=dataset_loader_name).version)

    _dataset_loader_version = schema.ObjectVersionInput(name=dataset_loader_name, version=dataset_loader_version)
    dataset_collector_schema = op.dataset_loader_version_from_name_version(dataset_loader_version=_dataset_loader_version).data_json_schema(
        collector_name=collector_name
    )
    dataset_collector_schema.dataset_loader_method_schema.schema_name()
    dataset_collector_schema.collector_method_schema.schema_name()

    dataset_loader_version_obj = send_graphql_request(op, json_response=False)

    dataset_loader_method_schema = (
        dataset_loader_version_obj.dataset_loader_version_from_name_version.data_json_schema.dataset_loader_method_schema.schema_name
    )
    collector_method_schema = (
        dataset_loader_version_obj.dataset_loader_version_from_name_version.data_json_schema.collector_method_schema.schema_name
    )

    dataset_loader_method_params = schema.MethodParamsInput(
        method_name=dataset_loader_method_schema,
        method_params=json.dumps(dataset_params),
    )

    collector_method_params = schema.MethodParamsInput(method_name=collector_method_schema, method_params=json.dumps(collector_params))

    data_params = schema.DataParamsInput(
        dataset_loader_method_params=dataset_loader_method_params,
        collector_method_params=collector_method_params,
    )

    if model_version is None:
        model_version = int(RegistryManager().get_latest_model_version(name=model_name).version)

    op = Operation(schema.Query)
    _model_version = schema.ObjectVersionInput(name=model_name, version=model_version)

    if job_executor_version is None:
        job_executor_version = int(RegistryManager().get_latest_executor_version(name=job_executor_name).version)

    executor_model_schema = op.model_version_from_name_version(model_version=_model_version).job_json_schema(
        executor_name=job_executor_name, executor_version=job_executor_version
    )
    executor_model_schema.executor_method_schema.schema_name()
    executor_model_schema.model_methods_schema.schema_name()

    model_version_obj = send_graphql_request(op, json_response=False)

    executor_method_schema = model_version_obj.model_version_from_name_version.job_json_schema.executor_method_schema.schema_name

    executor_method_params = schema.MethodParamsInput(method_name=executor_method_schema, method_params=json.dumps(executor_params))

    model_methods_params = []

    for item in model_params:
        for key in item:
            model_methods_params.append(schema.MethodParamsInput(method_name=key, method_params=json.dumps(item[key])))

    executor_model_params = schema.ExecutorModelParamsInput(
        executor_method_params=executor_method_params,
        model_methods_params=model_methods_params,
    )

    op = Operation(schema.Mutation)
    mutation = op.add_ml_job(
        job_executor_name=job_executor_name,
        job_executor_version=job_executor_version,
        dataset_loader_name=dataset_loader_name,
        dataset_loader_version=dataset_loader_version,
        model_name=model_name,
        model_version=model_version,
        new_model_name=new_model_name,
        experiment_name=experiment_name,
        collector_name=collector_name,
        gpu=gpu,
        data_params=data_params,
        executor_model_params=executor_model_params,
        choice_criteria=choice_criteria,
        cron_expression=cron_expression,
        periodic_type=periodic_type,
        metric_name=metric_name,
        optimal_min=optimal_min,
        force_rebuild=force_rebuild,
    )

    mutation.name()

    job = send_graphql_request(op, json_response=False)

    return job.add_ml_job.name


def job_by_name(name: str) -> Job:
    """
    Return Job object by name.

    Parameters
    ----------
    name: str
        Name of the job.

    Returns
    -------
    Job
        Instance of the Job class.
    """
    op = Operation(schema.Query)

    base_query = op.job_from_name(name=name)
    base_query.name()
    base_query.periodic_type()
    base_query.status()
    base_query.registration_timestamp()
    base_query.start_timestamp()
    base_query.end_timestamp()
    base_query.start_build_timestamp()
    base_query.end_build_timestamp()
    base_query.exception()

    job = send_graphql_request(op, json_response=False)

    return job.job_from_name


def job_metric_by_name(name: str) -> pd.DataFrame:
    """
    Job's most recent logged metrics.

    Parameters
    ----------
    name: str
        Name of the job.

    Returns
    -------
    pd.DataFrame
        Pandas dataframe with latest metrics.
    """
    op = Operation(schema.Query)

    op.job_from_name(name=name).run.latest_metrics()
    json_data = send_graphql_request(op)

    json_data = json_data["jobFromName"]["run"]["latestMetrics"]

    return pd.DataFrame([json_data])


def list_model_version(name: str) -> pd.DataFrame:
    """
    List available versions of the model with such name.

    Parameters
    ----------
    name: str
        Name of the model.

    Returns
    -------
    pd.DataFrame
        Pandas dataframe with a list of available model versions.
    """
    op = Operation(schema.Query)
    base_query = op.model_from_name(name=name).list_model_version
    base_query.version()
    base_query.creation_timestamp()
    base_query.status()
    json_data = send_graphql_request(op)

    df = pd.DataFrame.from_dict(json_data["modelFromName"]["listModelVersion"])
    df = _to_datetime(df, ["creationTimestamp"])

    return df.sort_values(by=["version"], ignore_index=True)


def list_dataset_loader_version(name: str) -> pd.DataFrame:
    """
    List available versions of the dataset_loader with such name.

    Parameters
    ----------
    name: str
        Name of the DatasetLoader.

    Returns
    -------
    pd.DataFrame
        Pandas dataframe with a list of available dataset_loader versions.
    """
    op = Operation(schema.Query)
    base_query = op.dataset_loader_from_name(name=name).list_dataset_loader_version
    base_query.version()
    base_query.creation_timestamp()
    base_query.status()
    json_data = send_graphql_request(op)

    df = pd.DataFrame.from_dict(json_data["datasetLoaderFromName"]["listDatasetLoaderVersion"])
    df = _to_datetime(df, ["creationTimestamp"])

    return df.sort_values(by=["version"], ignore_index=True)


def list_executor_version(name: str) -> pd.DataFrame:
    """
    List available versions of the executor with such name.

    Parameters
    ----------
    name: str
        Name of the executor.

    Returns
    -------
    pd.DataFrame
        Pandas dataframe with a list of available executor versions.
    """
    op = Operation(schema.Query)
    base_query = op.executor_from_name(name=name).list_executor_version
    base_query.version()
    base_query.creation_timestamp()
    base_query.status()
    json_data = send_graphql_request(op)

    df = pd.DataFrame.from_dict(json_data["executorFromName"]["listExecutorVersion"])
    df = _to_datetime(df, ["creationTimestamp"])

    return df.sort_values(by=["version"], ignore_index=True)


def model_version_metainfo(model_name: str, model_version: Optional[int] = None) -> ModelVersion:
    """
    Meta information about the model version by the model name and version.

    Parameters
    ----------
    model_name: str
        Name of the model.
    model_version: Optional[int] = None
        Version of the model. Default: None, "latest" version is used.

    Returns
    -------
    ModelVersion
        ModelVersion instance with meta information.
    """
    if model_version is None:
        model_version = int(RegistryManager().get_latest_model_version(name=model_name).version)

    op = Operation(schema.Query)
    _model_version = schema.ObjectVersionInput(name=model_name, version=model_version)
    base_query = op.model_version_from_name_version(model_version=_model_version)
    base_query.name()
    base_query.version()
    base_query.status()
    base_query.available_executor_versions.name()
    base_query.available_executor_versions.version()
    model_version = send_graphql_request(op, json_response=False)
    return model_version.model_version_from_name_version


def _generate_fake_schema(json_schema: dict) -> dict:

    if "required" not in json_schema.keys():
        return {}

    required_properties = {key: json_schema["properties"][key] for key in json_schema["required"]}
    json_schema["properties"] = required_properties

    faker = JSF(json_schema)
    fake_json = faker.generate()
    return fake_json


def _print_params_by_schema(json_schema: Dict, schema_type: str) -> None:
    """Print entity JSON Schema and example with required params."""
    properties_and_required_dict = {key: json_schema[key] for key in ("properties", "required") if key in json_schema}

    json_formatted_str = json.dumps(properties_and_required_dict, indent=2)

    print(f"{schema_type} json-schema:")

    print(json_formatted_str)

    print(f"{schema_type} parameters example:")

    fake_json = _generate_fake_schema(json_schema)

    print(fake_json)


def _get_model_schema_for_executor(model_name: str, model_version: int, executor_name: str, executor_version: int) -> dict:
    op = Operation(schema.Query)

    _model_version = schema.ObjectVersionInput(name=model_name, version=model_version)
    base_query = (
        op.model_version_from_name_version(model_version=_model_version)
        .job_json_schema(executor_name=executor_name, executor_version=executor_version)
        .model_methods_schema
    )

    base_query.schema_name()
    base_query.json_schema()

    json_data = send_graphql_request(op)
    return json_data["modelVersionFromNameVersion"]["jobJsonSchema"]["modelMethodsSchema"]


def print_model_schema_for_executor(
    model_name: str, executor_name: str, model_version: Optional[int] = None, executor_version: Optional[int] = None
) -> None:
    """
    Print particular model schema for particular executor.

    Parameters
    ----------
    model_name: str
        Name of the model.
    executor_name: str
        Name of the executor.
    model_version: Optional[int] = None
        Version of the model. Default: None, "latest" version is used.
    executor_version: Optional[int] = None
        Version of the executor. Default: None, "latest" version is used.
    """
    if model_version is None:
        model_version = int(RegistryManager().get_latest_model_version(name=model_name).version)

    if executor_version is None:
        executor_version = int(RegistryManager().get_latest_executor_version(name=executor_name).version)

    model_methods_schemas = _get_model_schema_for_executor(
        model_name=model_name, model_version=model_version, executor_name=executor_name, executor_version=executor_version
    )

    for model_methods_schema in model_methods_schemas:
        _print_params_by_schema(json.loads(model_methods_schema["jsonSchema"]), ModelMethodName(model_methods_schema["schemaName"]).name)


def generate_model_params_for_executor(
    model_name: str, executor_name: str, model_version: Optional[int] = None, executor_version: Optional[int] = None
) -> List[Dict[str, dict]]:
    """
    Return example of model's methods parameters for executor.

    Parameters
    ----------
    model_name: str
        Name of the model.
    executor_name: str
        Name of the executor.
    model_version: Optional[int] = None
        Version of the model. Default: None, "latest" version is used.
    executor_version: Optional[int] = None
        Version of the executor. Default: None, "latest" version is used.

    Returns
    -------
    List[Dict[str, Dict]]:
        Example of model's methods parameters. [{"method_one": {key1: value1, key2: value2}},
                                                {"method_two": {key1: value1, key2: value2}}]
    """
    if model_version is None:
        model_version = int(RegistryManager().get_latest_model_version(name=model_name).version)

    if executor_version is None:
        executor_version = int(RegistryManager().get_latest_executor_version(name=executor_name).version)

    model_methods_schemas = _get_model_schema_for_executor(
        model_name=model_name, model_version=model_version, executor_name=executor_name, executor_version=executor_version
    )

    list_model_params = []
    for model_methods_schema in model_methods_schemas:
        list_model_params.append(
            {model_methods_schema["schemaName"]: _generate_fake_schema(json.loads(model_methods_schema["jsonSchema"]))}
        )

    return list_model_params


def print_datasetloader_schema(name: str, version: Optional[int] = None) -> None:
    """
    Print DatasetLoader schema.

    Parameters
    ----------
    name: str
        Name of the DatasetLoader.
    version: Optional[int] = None
        Version of the DatasetLoader. Default: None, "latest" version is used.
    """
    if version is None:
        version = int(RegistryManager().get_latest_dataset_loader_version(name=name).version)

    op = Operation(schema.Query)
    _datasetloader_version = schema.ObjectVersionInput(name=name, version=version)
    base_query = op.dataset_loader_version_from_name_version(dataset_loader_version=_datasetloader_version)
    base_query.dataset_loader_method_schema()
    json_data = send_graphql_request(op)

    json_data = json.loads(json_data["datasetLoaderVersionFromNameVersion"]["datasetLoaderMethodSchema"])
    _print_params_by_schema(json_schema=json_data, schema_type="DatasetLoader")


def print_executor_schema(name: str, version: Optional[int] = None) -> None:
    """
    Print executor schema.

    Parameters
    ----------
    name: str
        Name of the executor.
    version: Optional[int] = None
        Version of the executor. Default: None, "latest" version is used.
    """
    if version is None:
        version = int(RegistryManager().get_latest_executor_version(name=name).version)
    op = Operation(schema.Query)
    executor_version = schema.ObjectVersionInput(name=name, version=version)
    base_query = op.executor_version_from_name_version(executor_version=executor_version)
    base_query.executor_method_schema()
    json_data = send_graphql_request(op)

    json_data = json.loads(json_data["executorVersionFromNameVersion"]["executorMethodSchema"])
    _print_params_by_schema(json_schema=json_data, schema_type="Executor")


def get_logs(job_name: str, file_name: Optional[str] = None) -> None:
    """
    Stream logs of the job by job name.

    Parameters
    ----------
    job_name: str
        Name of the job whose logs we want to view.
    file_name: Optional[str] = None
        Name of the file where to save logs. Default: None. If None prints logs to the output.
    """
    server_url = get_server_url()
    url_base = posixpath.join(server_url, "logs-api")
    local_path = f"filedump?job_id={job_name}"
    url = posixpath.join(url_base, local_path)
    with AuthSession().get(url, stream=True) as resp:
        if file_name:
            with open(file_name, "wb") as f:
                for line in resp.iter_lines():
                    if line:
                        f.writelines([line, b"\n\n"])
        else:
            for line in resp.iter_lines():
                if line:
                    print(line.decode())


def get_required_classes_by_executor(name: str, version: Optional[int] = None) -> List[str]:
    """
    Return the names of classes for the model to be inherited from, by the name of the executor.

    Parameters
    ----------
    name: str
        Name of the executor.
    executor_version: Optional[int] = None
        Version of the executor. Default: None, "latest" version is used.

    Returns
    -------
    List[str]:
        List of model class names to be inherited from.
    """
    if version is None:
        version = int(RegistryManager().get_latest_executor_version(name=name).version)
    op = Operation(schema.Query)
    executor_version = schema.ObjectVersionInput(name=name, version=version)
    base_query = op.executor_version_from_name_version(executor_version=executor_version)
    base_query.desired_model_patterns()
    json_data = send_graphql_request(op)
    return json_data["executorVersionFromNameVersion"]["desiredModelPatterns"]



"""Class inherits from base class of Model with a specific method for evaluating the model."""
from abc import ABC, abstractmethod

from ML_management.models.patterns import model_pattern


class EvaluatableModel(model_pattern.Model, ABC):
    """Implementation of evaluable model."""

    def __init__(self):
        super().__init__()

    @abstractmethod
    def evaluate_function(self, **kwargs):
        """Define evaluate function."""
        raise NotImplementedError



from ML_management.mlmanagement import mlmanagement

set_experiment = mlmanagement.set_experiment
get_experiment_by_name = mlmanagement.get_experiment_by_name
search_runs = mlmanagement.search_runs
start_run = mlmanagement.start_run
log_model = mlmanagement.log_model
log_metric = mlmanagement.log_metric
set_tag = mlmanagement.set_tag
autolog = mlmanagement.autolog
load_model = mlmanagement.load_model
save_model = mlmanagement.save_model
active_run = mlmanagement.active_run
get_run = mlmanagement.get_run
end_run = mlmanagement.end_run
log_artifact = mlmanagement.log_artifact
log_param = mlmanagement.log_param
log_params = mlmanagement.log_params
set_server_url = mlmanagement.set_server_url
get_server_url = mlmanagement.get_server_url
MlflowClient = mlmanagement.MlflowClient



"""Map supported executor function name to infer jsonschema to tag."""
from enum import Enum

from ML_management.executor_template import executor_pattern


class ExecutorMethodName(str, Enum):
    """Map supported executor function name to infer jsonschema."""

    execute = "executor_method_schema"


executor_pattern_to_methods = {executor_pattern.JobExecutorPattern: [ExecutorMethodName.execute]}
